@@ -24,7 +24,6 @@
 #include "appearance-font.h"
 #include "appearance-themes.h"
 #include "appearance-style.h"
-#include "appearance-ui.h"
 #include "theme-installer.h"
 #include "theme-thumbnail.h"
 #include "activate-settings-daemon.h"
@@ -39,6 +38,8 @@ init_appearance_data (int *argc, char ***argv, GOptionContext *context)
   GError *err = NULL;
 
   g_thread_init (NULL);
+  gdk_threads_init ();
+  gdk_threads_enter ();
   theme_thumbnail_factory_init (*argc, *argv);
   capplet_init (context, argc, argv);
   activate_settings_daemon ();
@@ -165,7 +166,6 @@ main (int argc, char **argv)
   desktop_init (data, (const gchar **) wallpaper_files);
   g_strfreev (wallpaper_files);
   font_init (data);
-  ui_init (data);
 
   /* prepare the main window */
   w = appearance_capplet_get_widget (data, "appearance_window");
@@ -209,6 +209,7 @@ main (int argc, char **argv)
 
   /* start the mainloop */
   gtk_main ();
+  gdk_threads_leave ();
 
   /* free stuff */
   g_free (data);
