@@ -78,12 +78,15 @@ struct App
 };
 
 static void rebuild_gui (App *app);
+static void on_clone_changed (GtkWidget *box, gpointer data);
 static void on_rate_changed (GtkComboBox *box, gpointer data);
 static gboolean output_overlaps (GnomeOutputInfo *output, GnomeRRConfig *config);
 static void select_current_output_from_dialog_position (App *app);
 static void monitor_on_off_toggled_cb (GtkToggleButton *toggle, gpointer data);
 static void get_geometry (GnomeOutputInfo *output, int *w, int *h);
 static void apply_configuration_returned_cb (DBusGProxy *proxy, DBusGProxyCall *call_id, void *data);
+static gboolean get_clone_size (GnomeRRScreen *screen, int *width, int *height);
+static gboolean output_info_supports_mode (App *app, GnomeOutputInfo *info, int width, int height);
 
 static void
 error_message (App *app, const char *primary_text, const char *secondary_text)
@@ -420,6 +423,67 @@ count_all_outputs (GnomeRRConfig *config)
 }
 #endif
 
+/* Computes whether "Mirror Screens" (clone mode) is supported based on these criteria:
+ *
+ * 1. There is an available size for cloning.
+ *
+ * 2. There are 2 or more connected outputs that support that size.
+ */
+static gboolean
+mirror_screens_is_supported (App *app)
+{
+    int clone_width, clone_height;
+    gboolean have_clone_size;
+    gboolean mirror_is_supported;
+
+    mirror_is_supported = FALSE;
+
+    have_clone_size = get_clone_size (app->screen, &clone_width, &clone_height);
+
+    if (have_clone_size) {
+	int i;
+	int num_outputs_with_clone_size;
+
+	num_outputs_with_clone_size = 0;
+
+	for (i = 0; app->current_configuration->outputs[i] != NULL; i++)
+	{
+	    GnomeOutputInfo *output = app->current_configuration->outputs[i];
+
+	    /* We count the connected outputs that support the clone size.  It
+	     * doesn't matter if those outputs aren't actually On currently; we
+	     * will turn them on in on_clone_changed().
+	     */
+	    if (output->connected && output_info_supports_mode (app, output, clone_width, clone_height))
+		num_outputs_with_clone_size++;
+	}
+
+	if (num_outputs_with_clone_size >= 2)
+	    mirror_is_supported = TRUE;
+    }
+
+    return mirror_is_supported;
+}
+
+static void
+rebuild_mirror_screens (App *app)
+{
+    gboolean mirror_is_active;
+    gboolean mirror_is_supported;
+
+    g_signal_handlers_block_by_func (app->clone_checkbox, G_CALLBACK (on_clone_changed), app);
+
+    mirror_is_active = app->current_configuration && app->current_configuration->clone;
+
+    /* If mirror_is_active, then it *must* be possible to turn mirroring off */
+    mirror_is_supported = mirror_is_active || mirror_screens_is_supported (app);
+    
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->clone_checkbox), mirror_is_active);
+    gtk_widget_set_sensitive (app->clone_checkbox, mirror_is_supported);
+
+    g_signal_handlers_unblock_by_func (app->clone_checkbox, G_CALLBACK (on_clone_changed), app);
+}
+
 static void
 rebuild_current_monitor_label (App *app)
 {
@@ -429,7 +493,11 @@ rebuild_current_monitor_label (App *app)
 
 	if (app->current_output)
 	{
-	    tmp = g_strdup_printf (_("Monitor: %s"), app->current_output->display_name);
+	    if (app->current_configuration->clone)
+		tmp = g_strdup (_("Mirror Screens"));
+	    else
+		tmp = g_strdup_printf (_("Monitor: %s"), app->current_output->display_name);
+
 	    str = g_strdup_printf ("<b>%s</b>", tmp);
 	    gnome_rr_labeler_get_color_for_output (app->labeler, app->current_output, &color);
 	    use_color = TRUE;
@@ -478,7 +546,11 @@ rebuild_on_off_radios (App *app)
     g_signal_handlers_block_by_func (app->monitor_on_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
     g_signal_handlers_block_by_func (app->monitor_off_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
 
-    if (app->current_output)
+    sensitive = FALSE;
+    on_active = FALSE;
+    off_active = FALSE;
+
+    if (!app->current_configuration->clone && app->current_output)
     {
 	if (count_active_outputs (app) > 1 || !app->current_output->on)
 	    sensitive = TRUE;
@@ -488,12 +560,6 @@ rebuild_on_off_radios (App *app)
 	on_active = app->current_output->on;
 	off_active = !on_active;
     }
-    else
-    {
-	sensitive = FALSE;
-	on_active = FALSE;
-	off_active = FALSE;
-    }
 
     gtk_widget_set_sensitive (app->monitor_on_radio, sensitive);
     gtk_widget_set_sensitive (app->monitor_off_radio, sensitive);
@@ -511,6 +577,29 @@ make_resolution_string (int width, int height)
     return g_strdup_printf (_("%d x %d"), width, height);
 }
 
+static void
+find_best_mode (GnomeRRMode **modes, int *out_width, int *out_height)
+{
+    int i;
+
+    *out_width = 0;
+    *out_height = 0;
+
+    for (i = 0; modes[i] != NULL; i++)
+    {
+	int w, h;
+
+	w = gnome_rr_mode_get_width (modes[i]);
+	h = gnome_rr_mode_get_height (modes[i]);
+
+	if (w * h > *out_width * *out_height)
+	{
+	    *out_width = w;
+	    *out_height = h;
+	}
+    }
+}
+
 static void
 rebuild_resolution_combo (App *app)
 {
@@ -548,7 +637,12 @@ rebuild_resolution_combo (App *app)
     current = idle_free (make_resolution_string (app->current_output->width, app->current_output->height));
 
     if (!combo_select (app->resolution_combo, current))
-	g_assert_not_reached ();
+    {
+	int best_w, best_h;
+
+	find_best_mode (modes, &best_w, &best_h);
+	combo_select (app->resolution_combo, idle_free (make_resolution_string (best_w, best_h)));
+    }
 }
 
 static void
@@ -569,6 +663,7 @@ rebuild_gui (App *app)
     g_debug ("rebuild gui, is on: %d", app->current_output->on);
 #endif
 
+    rebuild_mirror_screens (app);
     rebuild_current_monitor_label (app);
     rebuild_on_off_radios (app);
     rebuild_resolution_combo (app);
@@ -581,11 +676,6 @@ rebuild_gui (App *app)
     gtk_widget_set_sensitive (app->panel_checkbox, sensitive);
 
     app->ignore_gui_changes = FALSE;
-
-    if (app->current_configuration && app->current_configuration->clone)
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->clone_checkbox), TRUE);
-    else
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->clone_checkbox), FALSE);
 }
 
 static gboolean
@@ -653,29 +743,6 @@ on_rate_changed (GtkComboBox *box, gpointer data)
     foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
 }
 
-static void
-find_best_mode (GnomeRRMode **modes, int *out_width, int *out_height)
-{
-    int i;
-
-    *out_width = 0;
-    *out_height = 0;
-
-    for (i = 0; modes[i] != NULL; i++)
-    {
-	int w, h;
-
-	w = gnome_rr_mode_get_width (modes[i]);
-	h = gnome_rr_mode_get_height (modes[i]);
-
-	if (w * h > *out_width * *out_height)
-	{
-	    *out_width = w;
-	    *out_height = h;
-	}
-    }
-}
-
 static void
 select_resolution_for_current_output (App *app)
 {
@@ -853,6 +920,69 @@ lay_out_outputs_horizontally (App *app)
 
 }
 
+/* FIXME: this function is copied from gnome-settings-daemon/plugins/xrandr/gsd-xrandr-manager.c.
+ * Do we need to put this function in gnome-desktop for public use?
+ */
+static gboolean
+get_clone_size (GnomeRRScreen *screen, int *width, int *height)
+{
+        GnomeRRMode **modes = gnome_rr_screen_list_clone_modes (screen);
+        int best_w, best_h;
+        int i;
+
+        best_w = 0;
+        best_h = 0;
+
+        for (i = 0; modes[i] != NULL; ++i) {
+                GnomeRRMode *mode = modes[i];
+                int w, h;
+
+                w = gnome_rr_mode_get_width (mode);
+                h = gnome_rr_mode_get_height (mode);
+
+                if (w * h > best_w * best_h) {
+                        best_w = w;
+                        best_h = h;
+                }
+        }
+
+        if (best_w > 0 && best_h > 0) {
+                if (width)
+                        *width = best_w;
+                if (height)
+                        *height = best_h;
+
+                return TRUE;
+        }
+
+        return FALSE;
+}
+
+static gboolean
+output_info_supports_mode (App *app, GnomeOutputInfo *info, int width, int height)
+{
+    GnomeRROutput *output;
+    GnomeRRMode **modes;
+    int i;
+
+    if (!info->connected)
+	return FALSE;
+
+    output = gnome_rr_screen_get_output_by_name (app->screen, info->name);
+    if (!output)
+	return FALSE;
+
+    modes = gnome_rr_output_list_modes (output);
+
+    for (i = 0; modes[i]; i++) {
+	if (gnome_rr_mode_get_width (modes[i]) == width
+	    && gnome_rr_mode_get_height (modes[i]) == height)
+	    return TRUE;
+    }
+
+    return FALSE;
+}
+
 static void
 on_clone_changed (GtkWidget *box, gpointer data)
 {
@@ -864,6 +994,7 @@ on_clone_changed (GtkWidget *box, gpointer data)
     if (app->current_configuration->clone)
     {
 	int i;
+	int width, height;
 
 	for (i = 0; app->current_configuration->outputs[i]; ++i)
 	{
@@ -873,6 +1004,21 @@ on_clone_changed (GtkWidget *box, gpointer data)
 		break;
 	    }
 	}
+
+	/* Turn on all the connected screens that support the best clone mode.
+	 * The user may hit "Mirror Screens", but he shouldn't have to turn on
+	 * all the required outputs as well.
+	 */
+
+	get_clone_size (app->screen, &width, &height);
+
+	for (i = 0; app->current_configuration->outputs[i]; i++) {
+	    if (output_info_supports_mode (app, app->current_configuration->outputs[i], width, height)) {
+		app->current_configuration->outputs[i]->on = TRUE;
+		app->current_configuration->outputs[i]->width = width;
+		app->current_configuration->outputs[i]->height = height;
+	    }
+	}
     }
     else
     {
@@ -1318,6 +1464,43 @@ compare_snaps (gconstpointer v1, gconstpointer v2)
     }
 }
 
+/* Sets a mouse cursor for a widget's window.  As a hack, you can pass
+ * GDK_BLANK_CURSOR to mean "set the cursor to NULL" (i.e. reset the widget's
+ * window's cursor to its default).
+ */
+static void
+set_cursor (GtkWidget *widget, GdkCursorType type)
+{
+	GdkCursor *cursor;
+	GdkWindow *window;
+
+	if (type == GDK_BLANK_CURSOR)
+	    cursor = NULL;
+	else
+	    cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget), type);
+
+	window = gtk_widget_get_window (widget);
+
+	if (window)
+	    gdk_window_set_cursor (window, cursor);
+
+	if (cursor)
+	    gdk_cursor_unref (cursor);
+}
+
+static void
+set_monitors_tooltip (App *app, gboolean is_dragging)
+{
+    const char *text;
+
+    if (is_dragging)
+	text = NULL;
+    else
+	text = _("Select a monitor to change its properties; drag it to rearrange its placement.");
+
+    gtk_widget_set_tooltip_text (app->area, text);
+}
+
 static void
 on_output_event (FooScrollArea *area,
 		 FooScrollAreaEvent *event,
@@ -1326,6 +1509,13 @@ on_output_event (FooScrollArea *area,
     GnomeOutputInfo *output = data;
     App *app = g_object_get_data (G_OBJECT (area), "app");
 
+    /* If the mouse is inside the outputs, set the cursor to "you can move me".  See
+     * on_canvas_event() for where we reset the cursor to the default if it
+     * exits the outputs' area.
+     */
+    if (!app->current_configuration->clone && get_n_connected (app) > 1)
+	set_cursor (GTK_WIDGET (area), GDK_FLEUR);
+
     if (event->type == FOO_BUTTON_PRESS)
     {
 	GrabInfo *info;
@@ -1333,6 +1523,7 @@ on_output_event (FooScrollArea *area,
 	app->current_output = output;
 
 	rebuild_gui (app);
+	set_monitors_tooltip (app, TRUE);
 
 	if (!app->current_configuration->clone && get_n_connected (app) > 1)
 	{
@@ -1410,6 +1601,7 @@ on_output_event (FooScrollArea *area,
 	    if (event->type == FOO_BUTTON_RELEASE)
 	    {
 		foo_scroll_area_end_grab (area);
+		set_monitors_tooltip (app, FALSE);
 
 		g_free (output->user_data);
 		output->user_data = NULL;
@@ -1424,24 +1616,17 @@ on_output_event (FooScrollArea *area,
     }
 }
 
-#if 0
 static void
 on_canvas_event (FooScrollArea *area,
 		 FooScrollAreaEvent *event,
 		 gpointer data)
 {
-    App *app = g_object_get_data (G_OBJECT (area), "app");
-
-    if (event->type == FOO_BUTTON_PRESS)
-    {
-	app->current_output = NULL;
-
-	rebuild_gui (app);
-
-	foo_scroll_area_invalidate (area);
-    }
+    /* If the mouse exits the outputs, reset the cursor to the default.  See
+     * on_output_event() for where we set the cursor to the movement cursor if
+     * it is over one of the outputs.
+     */
+    set_cursor (GTK_WIDGET (area), GDK_BLANK_CURSOR);
 }
-#endif
 
 static PangoLayout *
 get_display_name (App *app,
@@ -1485,9 +1670,7 @@ paint_background (FooScrollArea *area,
 
     cairo_fill_preserve (cr);
 
-#if 0
     foo_scroll_area_add_input_from_fill (area, cr, on_canvas_event, NULL);
-#endif
 
     cairo_set_source_rgb (cr,
                           widget->style->dark[GTK_STATE_SELECTED].red / 65535.0,
@@ -2214,6 +2397,8 @@ run_application (App *app)
 
     g_object_set_data (G_OBJECT (app->area), "app", app);
 
+    set_monitors_tooltip (app, FALSE);
+
     /* FIXME: this should be computed dynamically */
     foo_scroll_area_set_min_size (FOO_SCROLL_AREA (app->area), -1, 200);
     gtk_widget_show (app->area);
