@@ -19,10 +19,10 @@
  */
 
 #include "document-tile.h"
+#include "config.h"
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
-#include <libgnomeui/gnome-icon-lookup.h>
 #include <gio/gio.h>
 
 #include "slab-gnome-util.h"
@@ -137,6 +137,7 @@ document_tile_new (const gchar *in_uri, const gchar *mime_type, time_t modified)
 	gchar *time_str;
 
 	gchar *markup;
+	gchar *str;
 
 	AtkObject *accessible;
 
@@ -189,11 +190,13 @@ document_tile_new (const gchar *in_uri, const gchar *mime_type, time_t modified)
 	/* make open with default action */
 
 	if (priv->default_app) {
-		markup = g_markup_printf_escaped (_("<b>Open with \"%s\"</b>"),
-						    g_app_info_get_name (priv->default_app));
+		str = g_strdup_printf (_("Open with \"%s\""),
+				       g_app_info_get_name (priv->default_app));
+		markup = g_markup_printf_escaped ("<b>%s</b>", str);
 		action = tile_action_new (TILE (this), open_with_default_trigger, markup,
 			TILE_ACTION_OPENS_NEW_WINDOW);
 		g_free (markup);
+		g_free (str);
 
 		TILE (this)->default_action = action;
 
@@ -413,7 +416,8 @@ load_image (DocumentTile *tile)
 
 	gchar *icon_id = NULL;
 	gboolean free_icon_id = TRUE;
-	GnomeThumbnailFactory *thumbnail_factory;
+	GnomeDesktopThumbnailFactory *thumbnail_factory;
+	GIcon *icon;
 
 	libslab_checkpoint ("document-tile.c: load_image(): start for %s", TILE (tile)->uri);
 
@@ -429,12 +433,14 @@ load_image (DocumentTile *tile)
 
 	thumbnail_factory = libslab_thumbnail_factory_get ();
 
-	icon_id = gnome_thumbnail_factory_lookup (thumbnail_factory, TILE (tile)->uri, priv->modified);
+	icon_id = gnome_desktop_thumbnail_factory_lookup (thumbnail_factory, TILE (tile)->uri, priv->modified);
 
-	if (! icon_id)
-		icon_id = gnome_icon_lookup (
-			gtk_icon_theme_get_default (), thumbnail_factory,
-			TILE (tile)->uri, NULL, NULL, priv->mime_type, 0, NULL);
+	if (! icon_id) {
+		icon = g_content_type_get_icon (priv->mime_type);
+		g_object_get (icon, "name", &icon_id, NULL);
+
+		g_object_unref (icon);
+	}	
 
 exit:
 
